@@ -289,13 +289,11 @@ typedef struct nvGpuRec {
     Bool allow_depth_30;
 
     char *name;  /* Name of the GPU */
-
+    
     unsigned int connected_displays;  /* Bitmask of connected displays */
 
-    int pci_bus;
-    int pci_device;
-    int pci_func;
-    
+    gchar *pci_bus_id;
+
     GvoModeData *gvo_mode_data; /* Information about GVO modes available */
     unsigned int num_gvo_modes;
 
@@ -465,11 +463,7 @@ GtkWidget*  ctk_display_layout_new       (NvCtrlAttributeHandle *,
                                           CtkConfig *,
                                           nvLayoutPtr, /* Layout to display */
                                           int,         /* Width of image */
-                                          int,         /* Height of image */
-                                          ctk_display_layout_selected_callback,
-                                          void *selected_callback_data,
-                                          ctk_display_layout_modified_callback,
-                                          void *modified_callback_data
+                                          int          /* Height of image */
                                           );
 
 void ctk_display_layout_update (CtkDisplayLayout *);
@@ -519,7 +513,8 @@ void ctk_display_layout_set_screen_metamode (CtkDisplayLayout *,
 void ctk_display_layout_add_screen_metamode (CtkDisplayLayout *, nvScreenPtr);
 void ctk_display_layout_delete_screen_metamode (CtkDisplayLayout *,
                                                 nvScreenPtr,
-                                                int metamode_idx);
+                                                int metamode_idx,
+                                                Bool reselect);
 
 void ctk_display_layout_disable_display (CtkDisplayLayout *ctk_object,
                                          nvDisplayPtr display);
@@ -527,6 +522,11 @@ void ctk_display_layout_disable_display (CtkDisplayLayout *ctk_object,
 void ctk_display_layout_set_advanced_mode (CtkDisplayLayout *ctk_object,
                                            int advanced_mode);
 
+void ctk_display_layout_register_callbacks(CtkDisplayLayout *ctk_object,
+                                           ctk_display_layout_selected_callback,
+                                           void *selected_callback_data,
+                                           ctk_display_layout_modified_callback,
+                                           void *modified_callback_data);
 
 
 G_END_DECLS
