@@ -214,7 +214,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_IMAGE_SETTINGS);
     MAKE_SIGNAL(NV_CTRL_XINERAMA_STEREO);
     MAKE_SIGNAL(NV_CTRL_BUS_RATE);
-    MAKE_SIGNAL(NV_CTRL_SHOW_SLI_HUD);
+    MAKE_SIGNAL(NV_CTRL_SHOW_SLI_VISUAL_INDICATOR);
     MAKE_SIGNAL(NV_CTRL_XV_SYNC_TO_DISPLAY);
     MAKE_SIGNAL(NV_CTRL_GVO_OVERRIDE_HW_CSC);
     MAKE_SIGNAL(NV_CTRL_GVO_COMPOSITE_TERMINATION);
@@ -235,6 +235,8 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_DYNAMIC_TWINVIEW);
     MAKE_SIGNAL(NV_CTRL_MULTIGPU_DISPLAY_OWNER);
     MAKE_SIGNAL(NV_CTRL_GPU_SCALING);
+    MAKE_SIGNAL(NV_CTRL_GPU_SCALING_DEFAULT_TARGET);
+    MAKE_SIGNAL(NV_CTRL_GPU_SCALING_DEFAULT_METHOD);
     MAKE_SIGNAL(NV_CTRL_FRONTEND_RESOLUTION);
     MAKE_SIGNAL(NV_CTRL_BACKEND_RESOLUTION);
     MAKE_SIGNAL(NV_CTRL_FLATPANEL_NATIVE_RESOLUTION);
@@ -300,7 +302,29 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_GPU_CORES);
     MAKE_SIGNAL(NV_CTRL_GPU_MEMORY_BUS_WIDTH);
     MAKE_SIGNAL(NV_CTRL_GVI_TEST_MODE);
-
+    MAKE_SIGNAL(NV_CTRL_COLOR_SPACE);
+    MAKE_SIGNAL(NV_CTRL_COLOR_RANGE);
+    MAKE_SIGNAL(NV_CTRL_DITHERING);
+    MAKE_SIGNAL(NV_CTRL_DITHERING_MODE);
+    MAKE_SIGNAL(NV_CTRL_DITHERING_DEPTH);
+    MAKE_SIGNAL(NV_CTRL_CURRENT_DITHERING);
+    MAKE_SIGNAL(NV_CTRL_CURRENT_DITHERING_MODE);
+    MAKE_SIGNAL(NV_CTRL_CURRENT_DITHERING_DEPTH);
+    MAKE_SIGNAL(NV_CTRL_THERMAL_SENSOR_READING);
+    MAKE_SIGNAL(NV_CTRL_THERMAL_SENSOR_PROVIDER);
+    MAKE_SIGNAL(NV_CTRL_THERMAL_SENSOR_TARGET);
+    MAKE_SIGNAL(NV_CTRL_SHOW_MULTIGPU_VISUAL_INDICATOR);
+    MAKE_SIGNAL(NV_CTRL_GPU_CURRENT_PROCESSOR_CLOCK_FREQS);
+    MAKE_SIGNAL(NV_CTRL_GVIO_VIDEO_FORMAT_FLAGS);
+    MAKE_SIGNAL(NV_CTRL_GPU_PCIE_MAX_LINK_SPEED);
+    MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_TRANSCEIVER_CHANNEL);
+    MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_TRANSCEIVER_MODE);
+    MAKE_SIGNAL(NV_CTRL_SYNCHRONOUS_PALETTE_UPDATES);
+    MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_TRANSCEIVER_CHANNEL_QUALITY);
+    MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_GLASSES_MISSED_SYNC_CYCLES);
+    MAKE_SIGNAL(NV_CTRL_GVO_ANC_PARITY_COMPUTATION);
+    MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_GLASSES_PAIR_EVENT);
+    MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_GLASSES_UNPAIR_EVENT);
 #undef MAKE_SIGNAL
     
     /*
@@ -310,7 +334,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
      * knows about.
      */
 
-#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_GVI_TEST_MODE
+#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_3D_VISION_PRO_GLASSES_UNPAIR_EVENT
 #warning "There are attributes that do not emit signals!"
 #endif
 
@@ -352,9 +376,11 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_SLI_MODE);
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_PERFORMANCE_MODES);
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_GVIO_VIDEO_FORMAT_NAME);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_GPU_CURRENT_CLOCK_FREQS);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_3D_VISION_PRO_GLASSES_NAME);
 #undef MAKE_STRING_SIGNAL
 
-#if NV_CTRL_STRING_LAST_ATTRIBUTE != NV_CTRL_STRING_GVO_VIDEO_FORMAT_NAME
+#if NV_CTRL_STRING_LAST_ATTRIBUTE != NV_CTRL_STRING_3D_VISION_PRO_GLASSES_NAME
 #warning "There are attributes that do not emit signals!"
 #endif
     
@@ -379,10 +405,11 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_VCSCS_USED_BY_GPU);
     MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_COOLERS_USED_BY_GPU);
     MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_GPUS_USED_BY_LOGICAL_XSCREEN);
+    MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_THERMAL_SENSORS_USED_BY_GPU);
 
 #undef MAKE_BINARY_SIGNAL
     
-#if NV_CTRL_BINARY_DATA_LAST_ATTRIBUTE != NV_CTRL_BINARY_DATA_GPUS_USED_BY_LOGICAL_XSCREEN
+#if NV_CTRL_BINARY_DATA_LAST_ATTRIBUTE != NV_CTRL_BINARY_DATA_GLASSES_PAIRED_TO_3D_VISION_PRO_TRANSCEIVER
 #warning "There are attributes that do not emit signals!"
 #endif
 
