@@ -522,7 +522,7 @@ static void setup_display_refresh_dropdown(CtkSLIMM *ctk_object)
         ctk_object->refresh_table_len = 0;
     }
     ctk_object->refresh_table =
-        (nvModeLinePtr *)calloc(ctk_object->num_modelines, sizeof(nvModeLinePtr));
+        calloc(ctk_object->num_modelines, sizeof(nvModeLinePtr));
     if (!ctk_object->refresh_table) {
         goto fail;
     }
@@ -694,8 +694,7 @@ static void setup_display_resolution_dropdown(CtkSLIMM *ctk_object)
         ctk_object->resolution_table_len = 0;
     }
     ctk_object->resolution_table =
-        (nvModeLinePtr *)calloc((ctk_object->num_modelines + 1),
-                                sizeof(nvModeLinePtr));
+        calloc((ctk_object->num_modelines + 1), sizeof(nvModeLinePtr));
     if (!ctk_object->resolution_table) {
         goto fail;
     }
@@ -859,7 +858,7 @@ static Bool parse_slimm_layout(CtkSLIMM *ctk_slimm,
                     gridConfigs[loc_idx].rows * gridConfigs[loc_idx].columns;
             }
         }
-        locs = (DpyLoc *)malloc(max_locs * sizeof(DpyLoc));
+        locs = malloc(max_locs * sizeof(DpyLoc));
         if (!locs) {
             err_msg = "Out of memory.";
             goto fail;
@@ -1049,8 +1048,11 @@ static Bool parse_slimm_layout(CtkSLIMM *ctk_slimm,
                 }
             }
         }
-        if (*voverlap > 0) { *voverlap = (*cur_modeline)->data.vdisplay - *voverlap; }
-        if (*voverlap < 0) { *voverlap += (*cur_modeline)->data.vdisplay; }
+        if (*voverlap > 0) {
+            *voverlap = (*cur_modeline)->data.vdisplay - *voverlap;
+        } else if (*voverlap < 0) {
+            *voverlap += (*cur_modeline)->data.vdisplay;
+        }
 
 
         /* Calculte column overlap */
@@ -1069,8 +1071,11 @@ static Bool parse_slimm_layout(CtkSLIMM *ctk_slimm,
                 }
             }
         }
-        if (*hoverlap > 0) { *hoverlap = (*cur_modeline)->data.hdisplay - *hoverlap; }
-        if (*hoverlap < 0) { *hoverlap += (*cur_modeline)->data.hdisplay; }
+        if (*hoverlap > 0) {
+            *hoverlap = (*cur_modeline)->data.hdisplay - *hoverlap;
+        } else if (*hoverlap < 0) {
+            *hoverlap += (*cur_modeline)->data.hdisplay;
+        }
     }
     
     XFree(metamode_str);
@@ -1242,6 +1247,8 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
     gchar *tmp;
     gchar *sli_mode = NULL;
     ReturnStatus ret;
+    ReturnStatus ret1;
+    int major = 0, minor = 0;
     gint val;
 
     nvLayoutPtr layout;
@@ -1252,6 +1259,8 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
     int count;
 
     Bool valid_layout = FALSE;
+    Bool trust_slimm_available = FALSE;
+    int vcs_target_count;
 
     int hoverlap = 0;
     int voverlap = 0;
@@ -1267,7 +1276,29 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
     ctk_slimm->handle = handle;
     ctk_slimm->ctk_config = ctk_config;
     ctk_object = ctk_slimm;
+
+    /*
+     * Check for NV-CONTROL protocol version.
+     * This used for not trust old X drivers which always reported
+     * it available (on NV50+).
+     */
+    ret = NvCtrlGetAttribute(handle,
+                             NV_CTRL_ATTR_NV_MAJOR_VERSION, &major);
+    ret1 = NvCtrlGetAttribute(handle,
+                              NV_CTRL_ATTR_NV_MINOR_VERSION, &minor);
+
+    if ((ret == NvCtrlSuccess) && (ret1 == NvCtrlSuccess) &&
+        ((major > 1) || ((major == 1) && (minor > 23)))) {
+        trust_slimm_available = TRUE;
+    }
     
+    vcs_target_count = ctk_config->pCtrlHandles->targets[VCS_TARGET].n;
+
+    /* return on old X drivers if target is other than VCS. */
+    if (!vcs_target_count && !trust_slimm_available) {
+      return NULL;
+    }
+
     /* Check if this screen supports SLI Mosaic Mode */
     ret = NvCtrlGetAttribute(ctk_object->handle,
                              NV_CTRL_SLI_MOSAIC_MODE_AVAILABLE, &val);
