@@ -46,7 +46,12 @@
  * 1.20        Added COOLER TargetType
  * 1.21        Added initial 64-bit integer attribute support (read-only)
  * 1.22        Added X_nvCtrlQueryValidStringAttributeValues to check
- *             string attribute permissions. 
+ *             string attribute permissions.
+ * 1.23        Added SENSOR TargetType
+ * 1.24        Fixed a bug where SLI_MOSAIC_MODE_AVAILABLE attribute would
+ *             report false positives via the GPU and X screen target types
+ * 1.25        Added 3D_VISION_PRO_TRANSCEIVER TargetType
+ * 1.26        Added XNVCTRLQueryXXXAttributePermissions.
  */
 
 #ifndef __NVCONTROL_H
@@ -57,7 +62,7 @@
 #define NV_CONTROL_NAME "NV-CONTROL"
 
 #define NV_CONTROL_MAJOR 1
-#define NV_CONTROL_MINOR 22
+#define NV_CONTROL_MINOR 26
 
 #define X_nvCtrlQueryExtension                      0
 #define X_nvCtrlIsNv                                1
@@ -66,8 +71,8 @@
 #define X_nvCtrlQueryStringAttribute                4
 #define X_nvCtrlQueryValidAttributeValues           5
 #define X_nvCtrlSelectNotify                        6
-#define X_nvCtrlSetGvoColorConversion_deprecated    7
-#define X_nvCtrlQueryGvoColorConversion_deprecated  8
+#define X_nvCtrlSetGvoColorConversionDeprecated     7
+#define X_nvCtrlQueryGvoColorConversionDeprecated   8
 #define X_nvCtrlSetStringAttribute                  9
 /* STUB X_nvCtrlQueryDDCCILutSize                   10 */
 /* STUB X_nvCtrlQueryDDCCISinglePointLutOperation   11 */
@@ -88,7 +93,12 @@
 #define X_nvCtrlQueryValidAttributeValues64         26
 #define X_nvCtrlQueryAttribute64                    27
 #define X_nvCtrlQueryValidStringAttributeValues     28
-#define X_nvCtrlLastRequest (X_nvCtrlQueryValidStringAttributeValues + 1)
+#define X_nvCtrlQueryAttributePermissions                29
+#define X_nvCtrlQueryStringAttributePermissions          30
+#define X_nvCtrlQueryBinaryDataAttributePermissions      31
+#define X_nvCtrlQueryStringOperationAttributePermissions 32
+
+#define X_nvCtrlLastRequest (X_nvCtrlQueryStringOperationAttributePermissions + 1)
 
 
 /* Define 32 bit floats */
@@ -333,6 +343,29 @@ typedef struct {
 #define sz_xnvCtrlQueryValidAttributeValues64Reply 48
 #define sz_xnvCtrlQueryValidAttributeValues64Reply_extra ((48 - 32) >> 2)
 
+typedef struct {
+    CARD8 reqType;
+    CARD8 nvReqType;
+    CARD16 length B16;
+    CARD32 attribute B32;
+} xnvCtrlQueryAttributePermissionsReq;
+#define sz_xnvCtrlQueryAttributePermissionsReq 8
+
+typedef struct {
+    BYTE type;
+    BYTE pad0;
+    CARD16 sequenceNumber B16;
+    CARD32 length B32;
+    CARD32 flags B32;
+    INT32 attr_type B32;
+    CARD32 perms B32;
+    CARD32 pad5 B32;
+    CARD32 pad6 B32;
+    CARD32 pad7 B32;
+    CARD32 pad8 B32;
+} xnvCtrlQueryAttributePermissionsReply;
+#define sz_xnvCtrlQueryAttributePermissionsReply 32
+
 /* Set GVO Color Conversion request (deprecated) */
 typedef struct {
     CARD8 reqType;
